"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var React = require('react');
var chai = require('chai');
var enzyme_1 = require('enzyme');
var redux_1 = require('redux');
var react_redux_1 = require('react-redux');
var assign = require('object-assign');
var graphql_1 = require('graphql');
var apollo_client_1 = require('apollo-client');
var chaiEnzyme = require('chai-enzyme');
chai.use(chaiEnzyme());
var expect = chai.expect;
var connect_1 = require('../src/connect');
describe('connect', function () {
    var Passthrough = (function (_super) {
        __extends(Passthrough, _super);
        function Passthrough() {
            _super.apply(this, arguments);
        }
        Passthrough.prototype.render = function () {
            return React.createElement("span", __assign({}, this.props));
        };
        return Passthrough;
    }(React.Component));
    ;
    var ProviderMock = (function (_super) {
        __extends(ProviderMock, _super);
        function ProviderMock(props, context) {
            _super.call(this, props, context);
            this.client = props.client;
            if (props.store) {
                this.store = props.store;
                return;
            }
            props.client.initStore();
            this.store = props.client.store;
        }
        ProviderMock.prototype.getChildContext = function () {
            return {
                store: this.store,
                client: this.client,
            };
        };
        ProviderMock.prototype.render = function () {
            var children = this.props.children;
            return React.Children.only(children);
        };
        ProviderMock.propTypes = {
            store: React.PropTypes.shape({
                subscribe: React.PropTypes.func.isRequired,
                dispatch: React.PropTypes.func.isRequired,
                getState: React.PropTypes.func.isRequired,
            }),
            client: React.PropTypes.object.isRequired,
            children: React.PropTypes.element.isRequired,
        };
        ProviderMock.childContextTypes = {
            store: React.PropTypes.object.isRequired,
            client: React.PropTypes.object.isRequired,
        };
        return ProviderMock;
    }(React.Component));
    ;
    describe('prop api', function () {
        it('should pass `ApolloClient.query` as props.query', function () {
            var store = redux_1.createStore(function () { return ({}); });
            var query = "\n        query people {\n          allPeople(first: 1) {\n            people {\n              name\n            }\n          }\n        }\n      ";
            var data = {
                allPeople: {
                    people: [
                        {
                            name: 'Luke Skywalker',
                        },
                    ],
                },
            };
            var networkInterface = mockNetworkInterface({
                request: { query: query },
                result: { data: data },
            });
            var client = new apollo_client_1.default({
                networkInterface: networkInterface,
            });
            var Container = (function (_super) {
                __extends(Container, _super);
                function Container() {
                    _super.apply(this, arguments);
                }
                Container.prototype.render = function () {
                    return React.createElement(Passthrough, __assign({}, this.props));
                };
                Container = __decorate([
                    connect_1.default()
                ], Container);
                return Container;
            }(React.Component));
            ;
            var wrapper = enzyme_1.mount(React.createElement(ProviderMock, {store: store, client: client}, React.createElement(Container, {pass: 'through', baz: 50})));
            var props = wrapper.find('span').props();
            expect(props.query).to.exist;
            expect(props.query({ query: query })).to.be.instanceof(Promise);
        });
        it('should pass `ApolloClient.mutate` as props.mutate', function () {
            var store = redux_1.createStore(function () { return ({}); });
            var client = new apollo_client_1.default();
            var Container = (function (_super) {
                __extends(Container, _super);
                function Container() {
                    _super.apply(this, arguments);
                }
                Container.prototype.render = function () {
                    return React.createElement(Passthrough, __assign({}, this.props));
                };
                Container = __decorate([
                    connect_1.default()
                ], Container);
                return Container;
            }(React.Component));
            ;
            var wrapper = enzyme_1.mount(React.createElement(ProviderMock, {store: store, client: client}, React.createElement(Container, {pass: 'through', baz: 50})));
            var props = wrapper.find('span').props();
            expect(props.mutate).to.exist;
            try {
                expect(props.mutate()).to.be.instanceof(Promise);
            }
            catch (e) {
                expect(e).to.be.instanceof(TypeError);
            }
            ;
        });
        it('should pass mutation methods as props.mutations dictionary', function () {
            var store = redux_1.createStore(function () { return ({}); });
            function mapMutationsToProps() {
                return {
                    test: function () { return ({
                        mutate: "",
                    }); },
                };
            }
            ;
            var Container = (function (_super) {
                __extends(Container, _super);
                function Container() {
                    _super.apply(this, arguments);
                }
                Container.prototype.render = function () {
                    return React.createElement(Passthrough, __assign({}, this.props));
                };
                Container = __decorate([
                    connect_1.default({ mapMutationsToProps: mapMutationsToProps })
                ], Container);
                return Container;
            }(React.Component));
            ;
            var wrapper = enzyme_1.mount(React.createElement(ProviderMock, {store: store, client: {}}, React.createElement(Container, {pass: 'through', baz: 50})));
            var props = wrapper.find('span').props();
            expect(props.mutations).to.exist;
            expect(props.mutations.test).to.exist;
            expect(props.mutations.test).to.be.instanceof(Function);
        });
        it('should rerun on prop change', function (done) {
            var store = redux_1.createStore(function () { return ({
                foo: 'bar',
                baz: 42,
                hello: 'world',
            }); });
            var query = "\n        query people {\n          allPeople(first: 1) {\n            people {\n              name\n            }\n          }\n        }\n      ";
            var data = {
                allPeople: {
                    people: [
                        {
                            name: 'Luke Skywalker',
                        },
                    ],
                },
            };
            var networkInterface = mockNetworkInterface({
                request: { query: query },
                result: { data: data },
            });
            var client = new apollo_client_1.default({
                networkInterface: networkInterface,
            });
            var run = false;
            function mapQueriesToProps(_a) {
                var ownProps = _a.ownProps;
                if (run) {
                    expect(ownProps.test).to.equal(null);
                    done();
                }
                if (ownProps.test === 'foo') {
                    run = true;
                }
                return {
                    people: { query: query },
                };
            }
            ;
            var Container = (function (_super) {
                __extends(Container, _super);
                function Container() {
                    _super.apply(this, arguments);
                }
                Container.prototype.render = function () {
                    return React.createElement(Passthrough, __assign({}, this.props));
                };
                Container = __decorate([
                    connect_1.default({ mapQueriesToProps: mapQueriesToProps })
                ], Container);
                return Container;
            }(React.Component));
            ;
            var ReRender = (function (_super) {
                __extends(ReRender, _super);
                function ReRender() {
                    _super.apply(this, arguments);
                    this.state = {
                        test: 'foo',
                    };
                }
                ReRender.prototype.componentDidMount = function () {
                    if (this.state.test === 'foo') {
                        this.setState({
                            test: null,
                        });
                    }
                };
                ReRender.prototype.render = function () {
                    return React.createElement(Container, __assign({}, this.props, {test: this.state.test}));
                };
                return ReRender;
            }(React.Component));
            ;
            enzyme_1.mount(React.createElement(ProviderMock, {store: store, client: client}, React.createElement(ReRender, null)));
        });
    });
    describe('redux passthrough', function () {
        it('should allow mapStateToProps', function () {
            var store = redux_1.createStore(function () { return ({
                foo: 'bar',
                baz: 42,
                hello: 'world',
            }); });
            var mapStateToProps = function (_a) {
                var foo = _a.foo, baz = _a.baz;
                return ({ foo: foo, baz: baz });
            };
            var Container = (function (_super) {
                __extends(Container, _super);
                function Container() {
                    _super.apply(this, arguments);
                }
                Container.prototype.render = function () {
                    return React.createElement(Passthrough, __assign({}, this.props));
                };
                Container = __decorate([
                    react_redux_1.connect(mapStateToProps)
                ], Container);
                return Container;
            }(React.Component));
            ;
            var ApolloContainer = (function (_super) {
                __extends(ApolloContainer, _super);
                function ApolloContainer() {
                    _super.apply(this, arguments);
                }
                ApolloContainer.prototype.render = function () {
                    return React.createElement(Passthrough, __assign({}, this.props));
                };
                ApolloContainer = __decorate([
                    connect_1.default({ mapStateToProps: mapStateToProps })
                ], ApolloContainer);
                return ApolloContainer;
            }(React.Component));
            ;
            var wrapper = enzyme_1.mount(React.createElement(ProviderMock, {store: store, client: {}}, React.createElement(Container, {pass: 'through', baz: 50})));
            var apolloWrapper = enzyme_1.mount(React.createElement(ProviderMock, {store: store, client: {}}, React.createElement(ApolloContainer, {pass: 'through', baz: 50})));
            var reduxProps = assign({}, wrapper.find('span').props(), {
                query: undefined,
                mutate: undefined,
            });
            var apolloProps = apolloWrapper.find('span').props();
            expect(reduxProps).to.deep.equal(apolloProps);
        });
        it('should allow mapDispatchToProps', function () {
            function doSomething(thing) {
                return {
                    type: 'APPEND',
                    body: thing,
                };
            }
            ;
            var store = redux_1.createStore(function () { return ({
                foo: 'bar',
                baz: 42,
                hello: 'world',
            }); });
            var mapDispatchToProps = function (dispatch) { return ({
                doSomething: function (whatever) { return dispatch(doSomething(whatever)); },
            }); };
            var Container = (function (_super) {
                __extends(Container, _super);
                function Container() {
                    _super.apply(this, arguments);
                }
                Container.prototype.render = function () {
                    return React.createElement(Passthrough, __assign({}, this.props));
                };
                Container = __decorate([
                    react_redux_1.connect(null, mapDispatchToProps)
                ], Container);
                return Container;
            }(React.Component));
            ;
            var ApolloContainer = (function (_super) {
                __extends(ApolloContainer, _super);
                function ApolloContainer() {
                    _super.apply(this, arguments);
                }
                ApolloContainer.prototype.render = function () {
                    return React.createElement(Passthrough, __assign({}, this.props));
                };
                ApolloContainer = __decorate([
                    connect_1.default({ mapDispatchToProps: mapDispatchToProps })
                ], ApolloContainer);
                return ApolloContainer;
            }(React.Component));
            ;
            var wrapper = enzyme_1.mount(React.createElement(ProviderMock, {store: store, client: {}}, React.createElement(Container, {pass: 'through', baz: 50})));
            var apolloWrapper = enzyme_1.mount(React.createElement(ProviderMock, {store: store, client: {}}, React.createElement(ApolloContainer, {pass: 'through', baz: 50})));
            var reduxProps = assign({}, wrapper.find('span').props(), {
                query: function () { },
                mutate: function () { },
            });
            var apolloProps = apolloWrapper.find('span').props();
            expect(reduxProps.doSomething()).to.deep.equal(apolloProps.doSomething());
        });
        it('should allow mergeProps', function () {
            function doSomething(thing) {
                return {
                    type: 'APPEND',
                    body: thing,
                };
            }
            ;
            var store = redux_1.createStore(function () { return ({
                foo: 'bar',
                baz: 42,
                hello: 'world',
            }); });
            var mapStateToProps = function (_a) {
                var foo = _a.foo, baz = _a.baz;
                return ({ foo: foo, baz: baz });
            };
            var mapDispatchToProps = function (dispatch) { return ({
                doSomething: function (whatever) { return dispatch(doSomething(whatever)); },
            }); };
            var mergeProps = function (stateProps, dispatchProps, ownProps) {
                return {
                    bar: stateProps.baz + 1,
                    makeSomething: dispatchProps.doSomething,
                    hallPass: ownProps.pass,
                };
            };
            var Container = (function (_super) {
                __extends(Container, _super);
                function Container() {
                    _super.apply(this, arguments);
                }
                Container.prototype.render = function () {
                    return React.createElement(Passthrough, __assign({}, this.props));
                };
                Container = __decorate([
                    react_redux_1.connect(mapStateToProps, mapDispatchToProps, mergeProps)
                ], Container);
                return Container;
            }(React.Component));
            ;
            var ApolloContainer = (function (_super) {
                __extends(ApolloContainer, _super);
                function ApolloContainer() {
                    _super.apply(this, arguments);
                }
                ApolloContainer.prototype.render = function () {
                    return React.createElement(Passthrough, __assign({}, this.props));
                };
                ApolloContainer = __decorate([
                    connect_1.default({ mapStateToProps: mapStateToProps, mapDispatchToProps: mapDispatchToProps, mergeProps: mergeProps })
                ], ApolloContainer);
                return ApolloContainer;
            }(React.Component));
            ;
            var wrapper = enzyme_1.mount(React.createElement(ProviderMock, {store: store, client: {}}, React.createElement(Container, {pass: 'through', baz: 50})));
            var apolloWrapper = enzyme_1.mount(React.createElement(ProviderMock, {store: store, client: {}}, React.createElement(ApolloContainer, {pass: 'through', baz: 50})));
            var reduxProps = assign({}, wrapper.find('span').props(), {
                query: function () { },
                mutate: function () { },
            });
            var apolloProps = apolloWrapper.find('span').props();
            expect(reduxProps.makeSomething()).to.deep.equal(apolloProps.makeSomething());
            expect(reduxProps.bar).to.equal(apolloProps.bar);
            expect(reduxProps.hallPass).to.equal(apolloProps.hallPass);
        });
    });
    describe('apollo methods', function () {
        describe('queries', function () {
            it('binds a query to props', function () {
                var store = redux_1.createStore(function () { return ({
                    foo: 'bar',
                    baz: 42,
                    hello: 'world',
                }); });
                var query = "\n          query people {\n            allPeople(first: 1) {\n              people {\n                name\n              }\n            }\n          }\n        ";
                var data = {
                    allPeople: {
                        people: [
                            {
                                name: 'Luke Skywalker',
                            },
                        ],
                    },
                };
                var networkInterface = mockNetworkInterface({
                    request: { query: query },
                    result: { data: data },
                });
                var client = new apollo_client_1.default({
                    networkInterface: networkInterface,
                });
                function mapQueriesToProps() {
                    return {
                        people: { query: query },
                    };
                }
                ;
                var Container = (function (_super) {
                    __extends(Container, _super);
                    function Container() {
                        _super.apply(this, arguments);
                    }
                    Container.prototype.render = function () {
                        return React.createElement(Passthrough, __assign({}, this.props));
                    };
                    Container = __decorate([
                        connect_1.default({ mapQueriesToProps: mapQueriesToProps })
                    ], Container);
                    return Container;
                }(React.Component));
                ;
                var wrapper = enzyme_1.mount(React.createElement(ProviderMock, {store: store, client: client}, React.createElement(Container, {pass: 'through', baz: 50})));
                var props = wrapper.find('span').props();
                expect(props.people).to.exist;
                expect(props.people.loading).to.be.true;
            });
            it('stops the query after unmounting', function () {
                var query = "\n          query people {\n            allPeople(first: 1) {\n              people {\n                name\n              }\n            }\n          }\n        ";
                var data = {
                    allPeople: {
                        people: [
                            {
                                name: 'Luke Skywalker',
                            },
                        ],
                    },
                };
                var networkInterface = mockNetworkInterface({
                    request: { query: query },
                    result: { data: data },
                });
                var client = new apollo_client_1.default({
                    networkInterface: networkInterface,
                });
                function mapQueriesToProps() {
                    return {
                        people: { query: query },
                    };
                }
                ;
                var Container = (function (_super) {
                    __extends(Container, _super);
                    function Container() {
                        _super.apply(this, arguments);
                    }
                    Container.prototype.render = function () {
                        return React.createElement(Passthrough, __assign({}, this.props));
                    };
                    Container = __decorate([
                        connect_1.default({ mapQueriesToProps: mapQueriesToProps })
                    ], Container);
                    return Container;
                }(React.Component));
                ;
                var wrapper = enzyme_1.mount(React.createElement(ProviderMock, {client: client}, React.createElement(Container, {pass: 'through', baz: 50})));
                var props = wrapper.find('span').props();
                expect(props.people).to.exist;
                expect(props.people.loading).to.be.true;
                expect(wrapper.unmount()).to.not.throw;
            });
            it('exposes refetch as part of the props api', function () {
                var store = redux_1.createStore(function () { return ({
                    foo: 'bar',
                    baz: 42,
                    hello: 'world',
                }); });
                var query = "\n          query people {\n            allPeople(first: 1) {\n              people {\n                name\n              }\n            }\n          }\n        ";
                var data = {
                    allPeople: {
                        people: [
                            {
                                name: 'Luke Skywalker',
                            },
                        ],
                    },
                };
                var networkInterface = mockNetworkInterface({
                    request: { query: query },
                    result: { data: data },
                });
                var client = new apollo_client_1.default({
                    networkInterface: networkInterface,
                });
                function mapQueriesToProps() {
                    return {
                        people: { query: query },
                    };
                }
                ;
                var Container = (function (_super) {
                    __extends(Container, _super);
                    function Container() {
                        _super.apply(this, arguments);
                    }
                    Container.prototype.render = function () {
                        return React.createElement(Passthrough, __assign({}, this.props));
                    };
                    Container = __decorate([
                        connect_1.default({ mapQueriesToProps: mapQueriesToProps })
                    ], Container);
                    return Container;
                }(React.Component));
                ;
                var wrapper = enzyme_1.mount(React.createElement(ProviderMock, {store: store, client: client}, React.createElement(Container, {pass: 'through', baz: 50})));
                var props = wrapper.find('span').props();
                expect(props.people).to.exist;
                expect(props.people.refetch).to.be.exist;
                expect(props.people.refetch).to.be.instanceof(Function);
                expect(props.people.refetch).to.not.throw;
            });
            it('resets the loading state when refetching', function (done) {
                var store = redux_1.createStore(function () { return ({
                    foo: 'bar',
                    baz: 42,
                    hello: 'world',
                }); });
                var query = "\n          query people {\n            allPeople(first: 1) {\n              people {\n                name\n              }\n            }\n          }\n        ";
                var data = {
                    allPeople: {
                        people: [
                            {
                                name: 'Luke Skywalker',
                            },
                        ],
                    },
                };
                var networkInterface = mockNetworkInterface({
                    request: { query: query },
                    result: { data: data },
                });
                var client = new apollo_client_1.default({
                    networkInterface: networkInterface,
                });
                function mapQueriesToProps() {
                    return {
                        people: { query: query },
                    };
                }
                ;
                var hasRefetched = false;
                var Container = (function (_super) {
                    __extends(Container, _super);
                    function Container() {
                        _super.apply(this, arguments);
                    }
                    Container.prototype.componentDidUpdate = function (prevProps) {
                        if (prevProps.people.loading && !this.props.people.loading) {
                            if (hasRefetched) {
                                return;
                            }
                            hasRefetched = true;
                            this.props.people.refetch();
                            return;
                        }
                        if (this.props.people.loading) {
                            expect(this.props.people.loading).to.be.true;
                            expect(this.props.people.result).to.exist;
                            done();
                        }
                    };
                    Container.prototype.render = function () {
                        return React.createElement(Passthrough, __assign({}, this.props));
                    };
                    Container = __decorate([
                        connect_1.default({ mapQueriesToProps: mapQueriesToProps })
                    ], Container);
                    return Container;
                }(React.Component));
                ;
                enzyme_1.mount(React.createElement(ProviderMock, {store: store, client: client}, React.createElement(Container, {pass: 'through', baz: 50})));
            });
            it('allows variables as part of the request', function () {
                var store = redux_1.createStore(function () { return ({
                    foo: 'bar',
                    baz: 42,
                    hello: 'world',
                }); });
                var query = "\n          query people($count: Int) {\n            allPeople(first: $count) {\n              people {\n                name\n              }\n            }\n          }\n        ";
                var variables = {
                    count: 1,
                };
                var data = {
                    allPeople: {
                        people: [
                            {
                                name: 'Luke Skywalker',
                            },
                        ],
                    },
                };
                var networkInterface = mockNetworkInterface({
                    request: { query: query, variables: variables },
                    result: { data: data },
                });
                var client = new apollo_client_1.default({
                    networkInterface: networkInterface,
                });
                function mapQueriesToProps() {
                    return {
                        people: { query: query, variables: variables },
                    };
                }
                ;
                var Container = (function (_super) {
                    __extends(Container, _super);
                    function Container() {
                        _super.apply(this, arguments);
                    }
                    Container.prototype.render = function () {
                        return React.createElement(Passthrough, __assign({}, this.props));
                    };
                    Container = __decorate([
                        connect_1.default({ mapQueriesToProps: mapQueriesToProps })
                    ], Container);
                    return Container;
                }(React.Component));
                ;
                var wrapper = enzyme_1.mount(React.createElement(ProviderMock, {store: store, client: client}, React.createElement(Container, {pass: 'through', baz: 50})));
                var props = wrapper.find('span').props();
                expect(props.people).to.exist;
                expect(props.people.loading).to.be.true;
            });
            it('can use passed props as part of the query', function () {
                var store = redux_1.createStore(function () { return ({
                    foo: 'bar',
                    baz: 42,
                    hello: 'world',
                }); });
                var query = "\n          query people($count: Int) {\n            allPeople(first: $count) {\n              people {\n                name\n              }\n            }\n          }\n        ";
                var variables = {
                    count: 1,
                };
                var data = {
                    allPeople: {
                        people: [
                            {
                                name: 'Luke Skywalker',
                            },
                        ],
                    },
                };
                var networkInterface = mockNetworkInterface({
                    request: { query: query, variables: variables },
                    result: { data: data },
                });
                var client = new apollo_client_1.default({
                    networkInterface: networkInterface,
                });
                function mapQueriesToProps(_a) {
                    var ownProps = _a.ownProps;
                    expect(ownProps.passedCountProp).to.equal(2);
                    return {
                        people: {
                            query: query,
                            variables: {
                                count: ownProps.passedCountProp,
                            },
                        },
                    };
                }
                ;
                var Container = (function (_super) {
                    __extends(Container, _super);
                    function Container() {
                        _super.apply(this, arguments);
                    }
                    Container.prototype.render = function () {
                        return React.createElement(Passthrough, __assign({}, this.props));
                    };
                    Container = __decorate([
                        connect_1.default({ mapQueriesToProps: mapQueriesToProps })
                    ], Container);
                    return Container;
                }(React.Component));
                ;
                var wrapper = enzyme_1.mount(React.createElement(ProviderMock, {store: store, client: client}, React.createElement(Container, {passedCountProp: 2})));
                var props = wrapper.find('span').props();
                expect(props.people).to.exist;
                expect(props.people.loading).to.be.true;
            });
            it('can use the redux state as part of the query', function () {
                var store = redux_1.createStore(function () { return ({
                    foo: 'bar',
                    baz: 42,
                    hello: 'world',
                }); });
                var query = "\n          query people($count: Int) {\n            allPeople(first: $count) {\n              people {\n                name\n              }\n            }\n          }\n        ";
                var variables = {
                    count: 1,
                };
                var data = {
                    allPeople: {
                        people: [
                            {
                                name: 'Luke Skywalker',
                            },
                        ],
                    },
                };
                var networkInterface = mockNetworkInterface({
                    request: { query: query, variables: variables },
                    result: { data: data },
                });
                var client = new apollo_client_1.default({
                    networkInterface: networkInterface,
                });
                function mapQueriesToProps(_a) {
                    var state = _a.state;
                    expect(state.hello).to.equal('world');
                    return {
                        people: {
                            query: query,
                            variables: {
                                count: 1,
                            },
                        },
                    };
                }
                ;
                var Container = (function (_super) {
                    __extends(Container, _super);
                    function Container() {
                        _super.apply(this, arguments);
                    }
                    Container.prototype.render = function () {
                        return React.createElement(Passthrough, __assign({}, this.props));
                    };
                    Container = __decorate([
                        connect_1.default({ mapQueriesToProps: mapQueriesToProps })
                    ], Container);
                    return Container;
                }(React.Component));
                ;
                var wrapper = enzyme_1.mount(React.createElement(ProviderMock, {store: store, client: client}, React.createElement(Container, {passedCountProp: 2})));
                var props = wrapper.find('span').props();
                expect(props.people).to.exist;
                expect(props.people.loading).to.be.true;
            });
            it('allows for multiple queries', function () {
                var store = redux_1.createStore(function () { return ({
                    foo: 'bar',
                    baz: 42,
                    hello: 'world',
                }); });
                var peopleQuery = "\n          query people($count: Int) {\n            allPeople(first: $count) {\n              people {\n                name\n              }\n            }\n          }\n        ";
                var peopleData = {
                    allPeople: {
                        people: [
                            {
                                name: 'Luke Skywalker',
                            },
                        ],
                    },
                };
                var shipQuery = "\n          query starships($count: Int) {\n            allStarships(first: $count) {\n              starships {\n                name\n              }\n            }\n          }\n        ";
                var variables = { count: 1 };
                var networkInterface = mockNetworkInterface({
                    request: { query: peopleQuery, variables: variables },
                    result: { data: peopleData },
                });
                var client = new apollo_client_1.default({
                    networkInterface: networkInterface,
                });
                function mapQueriesToProps() {
                    return {
                        people: { query: peopleQuery, variables: variables },
                        ships: { query: shipQuery, variables: variables },
                    };
                }
                ;
                var Container = (function (_super) {
                    __extends(Container, _super);
                    function Container() {
                        _super.apply(this, arguments);
                    }
                    Container.prototype.render = function () {
                        return React.createElement(Passthrough, __assign({}, this.props));
                    };
                    Container = __decorate([
                        connect_1.default({ mapQueriesToProps: mapQueriesToProps })
                    ], Container);
                    return Container;
                }(React.Component));
                ;
                var wrapper = enzyme_1.mount(React.createElement(ProviderMock, {store: store, client: client}, React.createElement(Container, null)));
                var props = wrapper.find('span').props();
                expect(props.people).to.exist;
                expect(props.people.loading).to.be.true;
                expect(props.ships).to.exist;
                expect(props.ships.loading).to.be.true;
            });
            it('should update the props of the child component when data is returned', function (done) {
                var store = redux_1.createStore(function () { return ({}); });
                var query = "\n          query people {\n            luke: allPeople(first: 1) {\n              people {\n                name\n              }\n            }\n          }\n        ";
                var data = {
                    luke: {
                        people: [
                            {
                                name: 'Luke Skywalker',
                            },
                        ],
                    },
                };
                var networkInterface = mockNetworkInterface({
                    request: { query: query },
                    result: { data: data },
                });
                var client = new apollo_client_1.default({
                    networkInterface: networkInterface,
                });
                function mapQueriesToProps() {
                    return {
                        luke: { query: query },
                    };
                }
                ;
                var Container = (function (_super) {
                    __extends(Container, _super);
                    function Container() {
                        _super.apply(this, arguments);
                    }
                    Container.prototype.componentDidUpdate = function (prevProps) {
                        expect(prevProps.luke.loading).to.be.true;
                        expect(this.props.luke.result).to.deep.equal(data);
                        done();
                    };
                    Container.prototype.render = function () {
                        return React.createElement(Passthrough, __assign({}, this.props));
                    };
                    Container = __decorate([
                        connect_1.default({ mapQueriesToProps: mapQueriesToProps })
                    ], Container);
                    return Container;
                }(React.Component));
                ;
                enzyme_1.mount(React.createElement(ProviderMock, {store: store, client: client}, React.createElement(Container, null)));
            });
            it('should prefill any data already in the store', function (done) {
                var query = "\n          query people {\n            allPeople(first: 1) {\n              people {\n                name\n              }\n            }\n          }\n        ";
                var data = {
                    allPeople: {
                        people: [
                            {
                                name: 'Luke Skywalker',
                            },
                        ],
                    },
                };
                var networkInterface = mockNetworkInterface({
                    request: { query: query },
                    result: { data: data },
                });
                var client = new apollo_client_1.default({
                    networkInterface: networkInterface,
                });
                var reducer = client.reducer();
                var store = redux_1.createStore(redux_1.combineReducers({
                    apollo: reducer,
                }), redux_1.applyMiddleware(client.middleware()));
                client.query({ query: query })
                    .then(function () {
                    function mapQueriesToProps() {
                        return {
                            people: { query: query },
                        };
                    }
                    ;
                    var Container = (function (_super) {
                        __extends(Container, _super);
                        function Container() {
                            _super.apply(this, arguments);
                        }
                        Container.prototype.render = function () {
                            return React.createElement(Passthrough, __assign({}, this.props));
                        };
                        Container = __decorate([
                            connect_1.default({ mapQueriesToProps: mapQueriesToProps })
                        ], Container);
                        return Container;
                    }(React.Component));
                    ;
                    var wrapper = enzyme_1.mount(React.createElement(ProviderMock, {store: store, client: client}, React.createElement(Container, {pass: 'through', baz: 50})));
                    var props = wrapper.find('span').props();
                    expect(props.people).to.exist;
                    expect(props.people.loading).to.be.false;
                    expect(props.people.result).to.deep.equal(data);
                    done();
                });
            });
        });
        describe('mutations', function () {
            it('should bind mutation data to props', function () {
                var store = redux_1.createStore(function () { return ({
                    foo: 'bar',
                    baz: 42,
                    hello: 'world',
                }); });
                var mutation = "\n          mutation makeListPrivate($listId: ID!) {\n            makeListPrivate(id: $listId)\n          }\n        ";
                var variables = {
                    listId: '1',
                };
                var data = {
                    makeListPrivate: true,
                };
                var networkInterface = mockNetworkInterface({
                    request: { query: mutation, variables: variables },
                    result: { data: data },
                });
                var client = new apollo_client_1.default({
                    networkInterface: networkInterface,
                });
                function mapMutationsToProps() {
                    return {
                        makeListPrivate: function () { return ({
                            mutation: mutation,
                            variables: variables,
                        }); },
                    };
                }
                ;
                var Container = (function (_super) {
                    __extends(Container, _super);
                    function Container() {
                        _super.apply(this, arguments);
                    }
                    Container.prototype.render = function () {
                        return React.createElement(Passthrough, __assign({}, this.props));
                    };
                    Container = __decorate([
                        connect_1.default({ mapMutationsToProps: mapMutationsToProps })
                    ], Container);
                    return Container;
                }(React.Component));
                ;
                var wrapper = enzyme_1.mount(React.createElement(ProviderMock, {store: store, client: client}, React.createElement(Container, null)));
                var props = wrapper.find('span').props();
                expect(props.makeListPrivate).to.exist;
                expect(props.makeListPrivate.loading).to.be.true;
            });
            it('should bind multiple mutation keys to props', function () {
                var store = redux_1.createStore(function () { return ({
                    foo: 'bar',
                    baz: 42,
                    hello: 'world',
                }); });
                var mutation1 = "\n          mutation makeListPrivate($listId: ID!) {\n            makeListPrivate(id: $listId)\n          }\n        ";
                var mutation2 = "\n          mutation makeListReallyPrivate($listId: ID!) {\n            makeListReallyPrivate(id: $listId)\n          }\n        ";
                var data = {
                    makeListPrivate: true,
                };
                var networkInterface = mockNetworkInterface({
                    request: { query: mutation1 },
                    result: { data: data },
                });
                var client = new apollo_client_1.default({
                    networkInterface: networkInterface,
                });
                function mapMutationsToProps() {
                    return {
                        makeListPrivate: function () { return ({
                            mutation1: mutation1,
                        }); },
                        makeListReallyPrivate: function () { return ({
                            mutation2: mutation2,
                        }); },
                    };
                }
                ;
                var Container = (function (_super) {
                    __extends(Container, _super);
                    function Container() {
                        _super.apply(this, arguments);
                    }
                    Container.prototype.render = function () {
                        return React.createElement(Passthrough, __assign({}, this.props));
                    };
                    Container = __decorate([
                        connect_1.default({ mapMutationsToProps: mapMutationsToProps })
                    ], Container);
                    return Container;
                }(React.Component));
                ;
                var wrapper = enzyme_1.mount(React.createElement(ProviderMock, {store: store, client: client}, React.createElement(Container, null)));
                var props = wrapper.find('span').props();
                expect(props.makeListPrivate).to.exist;
                expect(props.makeListPrivate.loading).to.be.true;
                expect(props.makeListReallyPrivate).to.exist;
                expect(props.makeListReallyPrivate.loading).to.be.true;
            });
            it('should bind mutation handler to `props.mutations[key]`', function () {
                var store = redux_1.createStore(function () { return ({
                    foo: 'bar',
                    baz: 42,
                    hello: 'world',
                }); });
                var mutation = "\n          mutation makeListPrivate($listId: ID!) {\n            makeListPrivate(id: $listId)\n          }\n        ";
                var variables = {
                    listId: '1',
                };
                var data = {
                    makeListPrivate: true,
                };
                var networkInterface = mockNetworkInterface({
                    request: { query: mutation, variables: variables },
                    result: { data: data },
                });
                var client = new apollo_client_1.default({
                    networkInterface: networkInterface,
                });
                function mapMutationsToProps() {
                    return {
                        makeListPrivate: function () { return ({
                            mutation: mutation,
                        }); },
                    };
                }
                ;
                var Container = (function (_super) {
                    __extends(Container, _super);
                    function Container() {
                        _super.apply(this, arguments);
                    }
                    Container.prototype.render = function () {
                        return React.createElement(Passthrough, __assign({}, this.props));
                    };
                    Container = __decorate([
                        connect_1.default({ mapMutationsToProps: mapMutationsToProps })
                    ], Container);
                    return Container;
                }(React.Component));
                ;
                var wrapper = enzyme_1.mount(React.createElement(ProviderMock, {store: store, client: client}, React.createElement(Container, null)));
                var props = wrapper.find('span').props();
                expect(props.makeListPrivate).to.exist;
                expect(props.makeListPrivate.loading).to.be.true;
                expect(props.mutations).to.exist;
                expect(props.mutations.makeListPrivate).to.exist;
                expect(props.mutations.makeListPrivate).to.be.instanceof(Function);
            });
            it('should pass the mutation promise to the child component', function (done) {
                var store = redux_1.createStore(function () { return ({
                    foo: 'bar',
                    baz: 42,
                    hello: 'world',
                }); });
                var mutation = "\n          mutation makeListPrivate($listId: ID!) {\n            makeListPrivate(id: $listId)\n          }\n        ";
                var variables = {
                    listId: '1',
                };
                var data = {
                    makeListPrivate: true,
                };
                var networkInterface = mockNetworkInterface({
                    request: { query: mutation, variables: variables },
                    result: { data: data },
                });
                var client = new apollo_client_1.default({
                    networkInterface: networkInterface,
                });
                function mapMutationsToProps() {
                    return {
                        makeListPrivate: function () { return ({
                            mutation: mutation,
                        }); },
                    };
                }
                ;
                var Container = (function (_super) {
                    __extends(Container, _super);
                    function Container() {
                        _super.apply(this, arguments);
                    }
                    Container.prototype.render = function () {
                        return React.createElement(Passthrough, __assign({}, this.props));
                    };
                    Container = __decorate([
                        connect_1.default({ mapMutationsToProps: mapMutationsToProps })
                    ], Container);
                    return Container;
                }(React.Component));
                ;
                var wrapper = enzyme_1.mount(React.createElement(ProviderMock, {store: store, client: client}, React.createElement(Container, null)));
                var props = wrapper.find('span').props();
                expect(props.makeListPrivate).to.exist;
                expect(props.makeListPrivate.loading).to.be.true;
                expect(props.mutations).to.exist;
                expect(props.mutations.makeListPrivate).to.exist;
                expect(props.mutations.makeListPrivate).to.be.instanceof(Function);
                props.mutations.makeListPrivate()
                    .then(function (err, result) {
                    done();
                })
                    .catch(function () {
                    done(new Error('should not error'));
                });
            });
            it('should update the props of the child component when data is returned', function (done) {
                var store = redux_1.createStore(function () { return ({}); });
                var mutation = "\n          mutation makeListPrivate($listId: ID!) {\n            makeListPrivate(id: $listId)\n          }\n        ";
                var variables = {
                    listId: '1',
                };
                var data = {
                    makeListPrivate: true,
                };
                var networkInterface = mockNetworkInterface({
                    request: { query: mutation, variables: variables },
                    result: { data: data },
                });
                var client = new apollo_client_1.default({
                    networkInterface: networkInterface,
                });
                function mapMutationsToProps() {
                    return {
                        makeListPrivate: function () { return ({
                            mutation: mutation,
                            variables: variables,
                        }); },
                    };
                }
                ;
                var Container = (function (_super) {
                    __extends(Container, _super);
                    function Container() {
                        _super.apply(this, arguments);
                    }
                    Container.prototype.componentDidMount = function () {
                        this.props.mutations.makeListPrivate();
                    };
                    Container.prototype.componentDidUpdate = function (prevProps) {
                        if (!this.props.makeListPrivate.loading) {
                            expect(prevProps.makeListPrivate.loading).to.be.true;
                            expect(this.props.makeListPrivate.result).to.deep.equal(data);
                            done();
                        }
                    };
                    Container.prototype.render = function () {
                        return React.createElement(Passthrough, __assign({}, this.props));
                    };
                    Container = __decorate([
                        connect_1.default({ mapMutationsToProps: mapMutationsToProps })
                    ], Container);
                    return Container;
                }(React.Component));
                ;
                enzyme_1.mount(React.createElement(ProviderMock, {store: store, client: client}, React.createElement(Container, null)));
            });
            it('can use passed props as part of the mutation', function (done) {
                var store = redux_1.createStore(function () { return ({
                    foo: 'bar',
                    baz: 42,
                    hello: 'world',
                }); });
                var mutation = "\n          mutation makeListPrivate($listId: ID!) {\n            makeListPrivate(id: $listId)\n          }\n        ";
                var variables = {
                    listId: '1',
                };
                var data = {
                    makeListPrivate: true,
                };
                var networkInterface = mockNetworkInterface({
                    request: { query: mutation, variables: variables },
                    result: { data: data },
                });
                var client = new apollo_client_1.default({
                    networkInterface: networkInterface,
                });
                function mapMutationsToProps(_a) {
                    var ownProps = _a.ownProps;
                    expect(ownProps.listId).to.equal('1');
                    return {
                        makeListPrivate: function () {
                            return {
                                mutation: mutation,
                                variables: {
                                    listId: ownProps.listId,
                                },
                            };
                        },
                    };
                }
                ;
                var Container = (function (_super) {
                    __extends(Container, _super);
                    function Container() {
                        _super.apply(this, arguments);
                    }
                    Container.prototype.componentDidMount = function () {
                        this.props.mutations.makeListPrivate();
                    };
                    Container.prototype.componentDidUpdate = function (prevProps) {
                        if (!this.props.makeListPrivate.loading) {
                            expect(prevProps.makeListPrivate.loading).to.be.true;
                            expect(this.props.makeListPrivate.result).to.deep.equal(data);
                            done();
                        }
                    };
                    Container.prototype.render = function () {
                        return React.createElement(Passthrough, __assign({}, this.props));
                    };
                    Container = __decorate([
                        connect_1.default({ mapMutationsToProps: mapMutationsToProps })
                    ], Container);
                    return Container;
                }(React.Component));
                ;
                enzyme_1.mount(React.createElement(ProviderMock, {store: store, client: client}, React.createElement(Container, {listId: '1'})));
            });
            it('can use the redux store as part of the mutation', function (done) {
                var store = redux_1.createStore(function () { return ({
                    foo: 'bar',
                    baz: 42,
                    listId: '1',
                }); });
                var mutation = "\n          mutation makeListPrivate($listId: ID!) {\n            makeListPrivate(id: $listId)\n          }\n        ";
                var variables = {
                    listId: '1',
                };
                var data = {
                    makeListPrivate: true,
                };
                var networkInterface = mockNetworkInterface({
                    request: { query: mutation, variables: variables },
                    result: { data: data },
                });
                var client = new apollo_client_1.default({
                    networkInterface: networkInterface,
                });
                function mapMutationsToProps(_a) {
                    var state = _a.state;
                    expect(state.listId).to.equal('1');
                    return {
                        makeListPrivate: function () {
                            return {
                                mutation: mutation,
                                variables: {
                                    listId: state.listId,
                                },
                            };
                        },
                    };
                }
                ;
                var Container = (function (_super) {
                    __extends(Container, _super);
                    function Container() {
                        _super.apply(this, arguments);
                    }
                    Container.prototype.componentDidMount = function () {
                        this.props.mutations.makeListPrivate();
                    };
                    Container.prototype.componentDidUpdate = function (prevProps) {
                        if (!this.props.makeListPrivate.loading) {
                            expect(prevProps.makeListPrivate.loading).to.be.true;
                            expect(this.props.makeListPrivate.result).to.deep.equal(data);
                            done();
                        }
                    };
                    Container.prototype.render = function () {
                        return React.createElement(Passthrough, __assign({}, this.props));
                    };
                    Container = __decorate([
                        connect_1.default({ mapMutationsToProps: mapMutationsToProps })
                    ], Container);
                    return Container;
                }(React.Component));
                ;
                enzyme_1.mount(React.createElement(ProviderMock, {store: store, client: client}, React.createElement(Container, {listId: '1'})));
            });
            it('should allow passing custom arugments to mutation handle', function (done) {
                var store = redux_1.createStore(function () { return ({}); });
                var mutation = "\n          mutation makeListPrivate($listId: ID!) {\n            makeListPrivate(id: $listId)\n          }\n        ";
                var variables = {
                    listId: '1',
                };
                var data = {
                    makeListPrivate: true,
                };
                var networkInterface = mockNetworkInterface({
                    request: { query: mutation, variables: variables },
                    result: { data: data },
                });
                var client = new apollo_client_1.default({
                    networkInterface: networkInterface,
                });
                function mapMutationsToProps() {
                    return {
                        makeListPrivate: function (listId) {
                            return {
                                mutation: mutation,
                                variables: {
                                    listId: listId,
                                },
                            };
                        },
                    };
                }
                ;
                var Container = (function (_super) {
                    __extends(Container, _super);
                    function Container() {
                        _super.apply(this, arguments);
                    }
                    Container.prototype.componentDidMount = function () {
                        this.props.mutations.makeListPrivate('1');
                    };
                    Container.prototype.componentDidUpdate = function (prevProps) {
                        if (!this.props.makeListPrivate.loading) {
                            expect(prevProps.makeListPrivate.loading).to.be.true;
                            expect(this.props.makeListPrivate.result).to.deep.equal(data);
                            done();
                        }
                    };
                    Container.prototype.render = function () {
                        return React.createElement(Passthrough, __assign({}, this.props));
                    };
                    Container = __decorate([
                        connect_1.default({ mapMutationsToProps: mapMutationsToProps })
                    ], Container);
                    return Container;
                }(React.Component));
                ;
                enzyme_1.mount(React.createElement(ProviderMock, {store: store, client: client}, React.createElement(Container, null)));
            });
        });
    });
});
function mockNetworkInterface(mockedRequest) {
    var requestToResultMap = {};
    var request = mockedRequest.request, result = mockedRequest.result;
    requestToResultMap[requestToKey(request)] = result;
    var queryMock = function (req) {
        return new Promise(function (resolve, reject) {
            var resultData = requestToResultMap[requestToKey(req)];
            if (!resultData) {
                throw new Error("Passed request that wasn't mocked: " + requestToKey(req));
            }
            resolve(resultData);
        });
    };
    return {
        query: queryMock,
        _uri: 'mock',
        _opts: {},
        _middlewares: [],
        use: function () { return; },
    };
}
function requestToKey(request) {
    var query = request.query && graphql_1.print(graphql_1.parse(request.query));
    return JSON.stringify({
        variables: request.variables,
        query: query,
    });
}
//# sourceMappingURL=connect.js.map